package javax.swing.text.html;

import java.awt.Polygon;
import java.util.StringTokenizer;
import javax.swing.text.AttributeSet;
import java.util.Vector;
import java.io.Serializable;

class Map implements Serializable
{
    private String name;
    private Vector<AttributeSet> areaAttributes;
    private Vector<Map.RegionContainment> areas;
    
    public Map() {
    }
    
    public Map(String name) {
        this.name = name;
    }
    
    public String getName() {
        return this.name;
    }
    
    public void addArea(AttributeSet set) {
        if (set == null) {
            return;
        }
        if (this.areaAttributes == null) {
            this.areaAttributes = new Vector(2);
        }
        this.areaAttributes.addElement(set.copyAttributes());
        return;
    }
    
    public void removeArea(AttributeSet set) {
        if (set != null && this.areaAttributes != null) {
            int n = (this.areas != null) ? this.areas.size() : 0;
            for (int i = this.areaAttributes.size() - 1; i >= 0; --i) {
                if (((AttributeSet)this.areaAttributes.elementAt(i)).isEqual(set)) {
                    this.areaAttributes.removeElementAt(i);
                    if (i < n) {
                        this.areas.removeElementAt(i);
                    }
                }
            }
        }
    }
    
    public AttributeSet[] getAreas() {
        int n = (this.areaAttributes != null) ? this.areaAttributes.size() : 0;
        if (n != 0) {
            AttributeSet[] anArray = new AttributeSet[n];
            this.areaAttributes.copyInto(anArray);
            return anArray;
        }
        return null;
    }
    
    public AttributeSet getArea(int n, int n2, int n3, int n4) {
        int initialCapacity = (this.areaAttributes != null) ? this.areaAttributes.size() : 0;
        if (initialCapacity > 0) {
            int n5 = (this.areas != null) ? this.areas.size() : 0;
            if (this.areas == null) {
                this.areas = new Vector(initialCapacity);
            }
            int i = 0;
            while (i < initialCapacity) {
                if (i >= n5) {
                    this.areas.addElement(this.createRegionContainment((AttributeSet)this.areaAttributes.elementAt(i)));
                }
                Map.RegionContainment regionContainment = (Map.RegionContainment)this.areas.elementAt(i);
                if (regionContainment != null && regionContainment.contains(n, n2, n3, n4)) {
                    return (AttributeSet)this.areaAttributes.elementAt(i);
                }
                ++i;
            }
        }
        return null;
    }
    
    protected Map.RegionContainment createRegionContainment(AttributeSet set) {
        Object attribute = set.getAttribute(HTML.Attribute.SHAPE);
        if (attribute == null) {
            attribute = "rect";
        }
        if ((attribute instanceof String)) {
            String lowerCase = ((String)attribute).toLowerCase();
            Object sharedInstance = null;
            try {
                if (lowerCase.equals("rect")) {
                    sharedInstance = new RectangleRegionContainment(set);
                }
                else if (lowerCase.equals("circle")) {
                    sharedInstance = new CircleRegionContainment(set);
                }
                else if (lowerCase.equals("poly")) {
                    sharedInstance = new PolygonRegionContainment(set);
                }
                else if (lowerCase.equals("default")) {
                    sharedInstance = Map.DefaultRegionContainment.sharedInstance();
                }
            }
            catch (RuntimeException ex) {
                sharedInstance = null;
            }
            return sharedInstance;
        }
        return null;
    }
    
    protected static int[] extractCoords(Object o) {
        if (o == null || !(o instanceof String)) {
            return null;
        }
        StringTokenizer stringTokenizer = new StringTokenizer((String)o, ", \t\n\r");
        int[] array = null;
        int n = 0;
        while (stringTokenizer.hasMoreElements()) {
            String s = stringTokenizer.nextToken();
            int n2;
            if (s.endsWith("%")) {
                n2 = -1;
                s = s.substring(0, s.length() - 1);
            }
            else {
                n2 = 1;
            }
            try {
                int int1 = Integer.parseInt(s);
                if (array == null) {
                    array = new int[4];
                }
                else if (n == array.length) {
                    int[] array2 = new int[array.length * 2];
                    System.arraycopy(array, 0, array2, 0, array.length);
                    array = array2;
                }
                array[n++] = int1 * n2;
            }
            catch (NumberFormatException ex) {
                return null;
            }
        }
        if (n > 0 && n != array.length) {
            int[] array3 = new int[n];
            System.arraycopy(array, 0, array3, 0, n);
            array = array3;
            return array;
        }
        return array;
    }
    
    static class RectangleRegionContainment implements Map.RegionContainment
    {
        float[] percents;
        int lastWidth;
        int lastHeight;
        int x0;
        int y0;
        int x1;
        int y1;
        
        public RectangleRegionContainment(AttributeSet set) {
            int[] coords = Map.extractCoords(set.getAttribute(HTML.Attribute.COORDS));
            this.percents = null;
            if (coords == null || coords.length != 4) {
                throw new RuntimeException("Unable to parse rectangular area");
            }
            this.x0 = coords[0];
            this.y0 = coords[1];
            this.x1 = coords[2];
            this.y1 = coords[3];
            if (this.x0 < 0 || this.y0 < 0 || this.x1 < 0 || this.y1 < 0) {
                this.percents = new float[4];
                int n = -1;
                this.lastHeight = n;
                this.lastWidth = n;
                for (int i = 0; i < 4; ++i) {
                    if (coords[i] < 0) {
                        this.percents[i] = (float)Math.abs(coords[i]) / 100.0f;
                    }
                    else {
                        this.percents[i] = -1.0f;
                    }
                }
            }
            return;
        }
        
        @Override
        public boolean contains(int n, int n2, int lastWidth, int lastHeight) {
            if (this.percents == null) {
                return this.contains(n, n2);
            }
            if (this.lastWidth != lastWidth || this.lastHeight != lastHeight) {
                this.lastWidth = lastWidth;
                this.lastHeight = lastHeight;
                if (this.percents[0] != -1.0f) {
                    this.x0 = (int)(this.percents[0] * (float)lastWidth);
                }
                if (this.percents[1] != -1.0f) {
                    this.y0 = (int)(this.percents[1] * (float)lastHeight);
                }
                if (this.percents[2] != -1.0f) {
                    this.x1 = (int)(this.percents[2] * (float)lastWidth);
                }
                if (this.percents[3] != -1.0f) {
                    this.y1 = (int)(this.percents[3] * (float)lastHeight);
                }
            }
            return this.contains(n, n2);
        }
        
        public boolean contains(int n, int n2) {
            return n >= this.x0 && n <= this.x1 && n2 >= this.y0 && n2 <= this.y1;
        }
    }
    
    static class PolygonRegionContainment extends Polygon implements Map.RegionContainment
    {
        float[] percentValues;
        int[] percentIndexs;
        int lastWidth;
        int lastHeight;
        
        public PolygonRegionContainment(AttributeSet set) {
            int[] coords = Map.extractCoords(set.getAttribute(HTML.Attribute.COORDS));
            if (coords == null || coords.length == 0 || coords.length % 2 != 0) {
                throw new RuntimeException("Unable to parse polygon area");
            }
            int n = 0;
            int n2 = -1;
            this.lastHeight = n2;
            this.lastWidth = n2;
            for (int i = coords.length - 1; i >= 0; --i) {
                if (coords[i] < 0) {
                    ++n;
                }
            }
            if (n > 0) {
                this.percentIndexs = new int[n];
                this.percentValues = new float[n];
                int j = coords.length - 1;
                int n3 = 0;
                while (j >= 0) {
                    if (coords[j] < 0) {
                        this.percentValues[n3] = (float)coords[j] / -100.0f;
                        this.percentIndexs[n3] = j;
                        ++n3;
                    }
                    --j;
                }
            }
            else {
                this.percentIndexs = null;
                this.percentValues = null;
            }
            this.npoints = coords.length / 2;
            this.xpoints = new int[this.npoints];
            this.ypoints = new int[this.npoints];
            for (int k = 0; k < this.npoints; ++k) {
                this.xpoints[k] = coords[k + k];
                this.ypoints[k] = coords[k + k + 1];
            }
            return;
        }
        
        @Override
        public boolean contains(int n, int n2, int lastWidth, int lastHeight) {
            if (this.percentValues == null || (this.lastWidth == lastWidth && this.lastHeight == lastHeight)) {
                return super.contains(n, n2);
            }
            this.bounds = null;
            this.lastWidth = lastWidth;
            this.lastHeight = lastHeight;
            float n3 = (float)lastWidth;
            float n4 = (float)lastHeight;
            for (int i = this.percentValues.length - 1; i >= 0; --i) {
                if (this.percentIndexs[i] % 2 == 0) {
                    this.xpoints[this.percentIndexs[i] / 2] = (int)(this.percentValues[i] * n3);
                }
                else {
                    this.ypoints[this.percentIndexs[i] / 2] = (int)(this.percentValues[i] * n4);
                }
            }
            return super.contains(n, n2);
        }
    }
    
    static class CircleRegionContainment implements Map.RegionContainment
    {
        int x;
        int y;
        int radiusSquared;
        float[] percentValues;
        int lastWidth;
        int lastHeight;
        
        public CircleRegionContainment(AttributeSet set) {
            int[] coords = Map.extractCoords(set.getAttribute(HTML.Attribute.COORDS));
            if (coords == null || coords.length != 3) {
                throw new RuntimeException("Unable to parse circular area");
            }
            this.x = coords[0];
            this.y = coords[1];
            this.radiusSquared = coords[2] * coords[2];
            if (coords[0] < 0 || coords[1] < 0 || coords[2] < 0) {
                int n = -1;
                this.lastHeight = n;
                this.lastWidth = n;
                this.percentValues = new float[3];
                for (int i = 0; i < 3; ++i) {
                    if (coords[i] < 0) {
                        this.percentValues[i] = (float)coords[i] / -100.0f;
                    }
                    else {
                        this.percentValues[i] = -1.0f;
                    }
                }
            }
            else {
                this.percentValues = null;
            }
            return;
        }
        
        @Override
        public boolean contains(int n, int n2, int a, int b) {
            if (this.percentValues != null && (this.lastWidth != a || this.lastHeight != b)) {
                int n3 = Math.min(a, b) / 2;
                this.lastWidth = a;
                this.lastHeight = b;
                if (this.percentValues[0] != -1.0f) {
                    this.x = (int)(this.percentValues[0] * (float)a);
                }
                if (this.percentValues[1] != -1.0f) {
                    this.y = (int)(this.percentValues[1] * (float)b);
                }
                if (this.percentValues[2] != -1.0f) {
                    this.radiusSquared = (int)(this.percentValues[2] * (float)Math.min(a, b));
                    this.radiusSquared *= this.radiusSquared;
                }
            }
            return (n - this.x) * (n - this.x) + (n2 - this.y) * (n2 - this.y) <= this.radiusSquared;
        }
    }
    
    static class DefaultRegionContainment implements Map.RegionContainment
    {
        static Map.DefaultRegionContainment si;
        
        public static Map.DefaultRegionContainment sharedInstance() {
            if (Map.DefaultRegionContainment.si == null) {
                Map.DefaultRegionContainment.si = new Map.DefaultRegionContainment();
            }
            return Map.DefaultRegionContainment.si;
        }
        
        @Override
        public boolean contains(int n, int n2, int n3, int n4) {
            return n <= n3 && n >= 0 && n2 >= 0 && n2 <= n3;
        }
        
        static {
            Map.DefaultRegionContainment.si = null;
        }
    }
    
    interface RegionContainment
    {
        public abstract boolean contains(int p0, int p1, int p2, int p3);
    }
}
